/*========================================================================
 * Copyright (c) 2009 Tampere University of Technology, Finland
 * 
 * Permission is hereby granted, free of charge, to any person 
 * obtaining a copy of this software and associated documentation 
 * files (the "Software"), to deal in the Software without restriction, 
 * including without limitation the rights to use, copy, modify, merge, 
 * publish, distribute, sublicense, and/or sell copies of the Software, 
 * and to permit persons to whom the Software is furnished to do so, 
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *======================================================================*/

// Flickr feeds
// RSS feed from Flick with tags
// show picture slideshow

function tr(s) { return s; }

function Image(parent) {
	this.antialiased = true;	
}

Image.prototype = new QWidget();


function FlickrWidget(parent)
{
    QWidget.call(this, parent);

	this.flickrUrl = "http://api.flickr.com/services/feeds/photos_public.gne?format=rss2"
	this.imageUrls = new Array();

	this.tagsLabel = new QLabel(tr("Tags:"),this);
	this.lineEdit = new QLineEdit(this);
	this.lineEdit["returnPressed"].connect(this, this.tagsUpdated);
	this.imageLabel = new QLabel(this);
	this.imageLabel.setSizePolicy(QSizePolicy.Ignored, QSizePolicy.Ignored);
    //this.imageLabel.scaledContents = true;
    this.imageLabel.alignment = Qt.AlignCenter;
    
    this.goButton = new QPushButton(tr("Go"),this);
    this.goButton["clicked"].connect(this, this.tagsUpdated);

	this.imagePixmap = new QPixmap();
	this.nextPixmap = new QPixmap();

	this.timer = new QTimer(this);
	this.timer["timeout"].connect(this, this.timerTimeout);
	
	this.rotTimer = new QTimer(this);
	this.rotTimer["timeout"].connect(this, this.rotTimerTimeout);
	this.angle = 90;

	var hBoxLayout = new QHBoxLayout();
	hBoxLayout.addWidget(this.tagsLabel,0,0);
	hBoxLayout.addWidget(this.lineEdit,0,0);
	hBoxLayout.addWidget(this.goButton,0,0);
	
	this.layout = new QVBoxLayout(this);
	this.layout.addLayout(hBoxLayout);
	this.layout.addWidget(this.imageLabel,Qt.AlignCenter,0);
	
	this.layout.setContentsMargins(0,0,0,0);

    this.windowTitle = tr("Flickr Widget");
    
    //this.setWindowState(Qt.WindowFullScreen);
}

FlickrWidget.prototype = new QWidget();

FlickrWidget.prototype.tagsUpdated = function() {
	this.loadFeed();
	
}

FlickrWidget.prototype.timerTimeout = function() {
	this.showRandomImage();
}

FlickrWidget.prototype.rotTimerTimeout = function() {
	if(this.angle == 450) {
		this.angle = 90;
	}
	var trans = new QTransform();
	//trans.rotate(this.angle,Qt.XAxis);
	trans.rotate(this.angle,Qt.YAxis);
	trans.rotate(this.angle,Qt.ZAxis);
	
	if(this.angle == 90 || this.angle == 270) {
		this.imagePixmap = new QPixmap(this.nextPixmap);
	}
	this.angle = this.angle + 1;
	this.imageLabel.setPixmap(this.imagePixmap.transformed(trans,Qt.FastTransformation));
}

FlickrWidget.prototype.showRandomImage = function() {
	if(this.imageUrls.length != 0) {
		this.timer.stop();
		var accessManager = new QNetworkAccessManager(this);
		accessManager["finished(QNetworkReply*)"].connect(this, this.imageReplyFinished);
		accessManager.get(new QNetworkRequest((new QUrl(this.imageUrls[Math.floor((this.imageUrls.length-2)*Math.random())]))));
	}
}

FlickrWidget.prototype.imageReplyFinished = function(reply) {
	var data = reply.readAll();
	this.nextPixmap.loadFromData(data, 0, Qt.AutoColor);
	if(!this.nextPixmap.isNull()) {
		this.nextPixmap = this.nextPixmap.scaled(this.imageLabel.width*0.8,this.imageLabel.height*0.8,Qt.KeepAspectRatio,Qt.SmoothTransformation);
		//this.imageLabel.setPixmap(this.imagePixmap);
		this.rotTimer.start(30);
	}
	else {
		/* imagePixmap was null, image data was not available (bad url)*/
		this.imageLabel.text = '<center>No image available</center>';
	}
	this.timer.start(5000);
}


FlickrWidget.prototype.loadFeed = function() {
	this.imageUrls = [];
	var url = this.flickrUrl + "&tags=" + this.lineEdit.text;
	var accessManager = new QNetworkAccessManager(this);
	accessManager["finished(QNetworkReply*)"].connect(this, flickrReplyFinished);
	accessManager.get(new QNetworkRequest(new QUrl(url)));
	this.timer.start(5000);
}

flickrReplyFinished = function(reply) {
	var xml = new QXmlStreamReader();
	xml.addData(reply.readAll());
	
	while (!xml.atEnd()) {
        xml.readNext();
        
        // We are at the start of an element
        if(xml.isStartElement()) {
            if(xml.name() == "enclosure") {
            	this.imageUrls.push(xml.attributes().value("url").toString());
			}
		} 
        // We are at the end of an element
        else if (xml.isEndElement()) {
        } 
        // Collecting items we would propably want to save in the tree
        else if (xml.isCharacters() && !xml.isWhitespace()) {
        }
    }
    if (xml.hasError() && (xml.error() != QXmlStreamReader.PrematureEndOfDocumentError)) {
        print("XML ERROR:", xml.lineNumber() + ":", xml.errorString());
    }
}

/* To use the application in standalone mode, remove the comment around this code
var wdgt = new FlickrWidget();
wdgt.show();

QCoreApplication.exec();
*/
