/*========================================================================
 * Copyright (c) 2009 Tampere University of Technology, Finland
 * 
 * Permission is hereby granted, free of charge, to any person 
 * obtaining a copy of this software and associated documentation 
 * files (the "Software"), to deal in the Software without restriction, 
 * including without limitation the rights to use, copy, modify, merge, 
 * publish, distribute, sublicense, and/or sell copies of the Software, 
 * and to permit persons to whom the Software is furnished to do so, 
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.  
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *======================================================================*/

/*
 * WeatherWidget.js
 * A simple weather widget obtaining weather data from the network.
 * Shows the current weather conditions and a forecast for the following three 
 * days. The weather data is obtained from the web service of BBC.
*/

function tr(s) { 
	return s; 
}

function WeatherWidget(parent) {
    QWidget.call(this, parent);

	// Set basic style settings for the widget (This does not possibly work in OS X yet!)
	this.styleSheet = "QComboBox { background-color: lightblue; font: 14px} QPushButton { background-color: lightblue; }";

	// a combobox for choosing the location
    this.locComboBox = new QComboBox();
    // An action is connected with the QComboBox
    this.locComboBox["activated(int)"].connect(this, "locChanged");
    // Add a set of hardcoded towns
    this.locComboBox.addItem(tr("San Francisco, California"));
    this.locComboBox.addItem(tr("Tampere, Finland"));
    this.locComboBox.addItem(tr("London, United Kingdom"));
    this.locComboBox.addItem(tr("Berlin, Germany"));
    this.locComboBox.addItem(tr("Barcelona, Spain"));
    this.locComboBox.addItem(tr("Beijing, China"));
    this.locComboBox.addItem(tr("Kolkata, India"));
    this.locComboBox.addItem(tr("Cape Town, South Africa"));
    this.locComboBox.addItem(tr("Moscow, Russia"));
    this.locComboBox.addItem(tr("Rio De Janeiro, Brazil"));
    this.locComboBox.addItem(tr("Oslo, Norway"));
    this.locComboBox.setCurrentIndex(1); // Select Tampere as the default.
    this.locComboBox.activated(1); // Activate the combobox to fetch the data.

	// Weather description shown as a weather icon (in a QLabel)
    this.descrLbl = new QLabel();
	// Place for the temperature shown with a large font in a QLabel.
    this.tempLbl = new QLabel();
	// Show the other weather data in a QLabel.
	this.weatherDataLbl = new QLabel();
	// To show the names of the days, weather images and temperatures
	this.forecDayLbl = new QLabel();
	this.forecDayLbl2 = new QLabel();
	this.forecDayLbl3 = new QLabel();	
	this.forecDescrLbl = new QLabel();
	this.forecDescrLbl2 = new QLabel();
	this.forecDescrLbl3 = new QLabel();
	this.forecTempLbl = new QLabel();
	this.forecTempLbl2 = new QLabel();
	this.forecTempLbl3 = new QLabel();
	this.copyrightLbl = new QLabel();
	this.accessManager = new QNetworkAccessManager();
    this.bbcLogo = new QPixmap();
	this.copyrightLbl = new QLabel();

   var reqReply = this.accessManager.get(new QNetworkRequest(new QUrl("http://lively.cs.tut.fi/images/bbc-logo.jpg")));
    function addImg() {
		var data = reqReply.readAll();
		this.bbcLogo.loadFromData(data, 0, Qt.AutoColor);
		this.bbcLogo = this.bbcLogo.scaled(58, 44, Qt.KeepAspectRatio, Qt.SmoothTransformation);
		this.copyrightLbl.setPixmap(this.bbcLogo);
    }
    reqReply.finished.connect(this, addImg);

    // Main layout where the widgets are added (respective column and row 
    var mainLayout = new QGridLayout(this);
    // The QComboBox for choosing the location
    mainLayout.addWidget(this.locComboBox, 0, 0, rowspan=1, columnspan=3, Qt.AlignLeft); 
	mainLayout.addWidget(new QLabel(tr("<font size=3>Current weather:</font>")), 1, 0, rowspan=1, columnspan=3, Qt.AlignLeft);
    // Weather icon
	mainLayout.addWidget(this.descrLbl, 2, 0, rowspan=1, columnspan=1, Qt.AlignLeft);
	// The current temperature value shown beside the weather icon 
	mainLayout.addWidget(this.tempLbl, 2, 1, rowspan=1, columnspan=1, Qt.AlignRight); 
	// new layout for the fields of the weather forecast
	var forecastLayout = new QGridLayout();
	// Add the layouts to the main one.
	mainLayout.addLayout(forecastLayout, 3, 0, rowspan=4, columnspan=3, Qt.AlignLeft);

	// QLabels for showing the forecast data for the following three days
	forecastLayout.addWidget(new QLabel(tr("<font size=2>Weather forecast for the following three days: </font>")), 0, 0, rowspan=1, columnspan=3, Qt.AlignLeft);
	forecastLayout.addWidget(this.forecDayLbl, 1, 0, rowspan=1, columnspan=1, Qt.AlignLeft);
	forecastLayout.addWidget(this.forecDayLbl2, 1, 1, rowspan=1, columnspan=1, Qt.AlignCenter);
	forecastLayout.addWidget(this.forecDayLbl3, 1, 2, rowspan=1, columnspan=1, Qt.AlignRight);
	forecastLayout.addWidget(this.forecDescrLbl, 2, 0, rowspan=1, columnspan=1, Qt.AlignLeft);
	forecastLayout.addWidget(this.forecDescrLbl2, 2, 1, rowspan=1, columnspan=1, Qt.AlignCenter);
	forecastLayout.addWidget(this.forecDescrLbl3, 2, 2, rowspan=1, columnspan=1, Qt.AlignRight);
	forecastLayout.addWidget(this.forecTempLbl, 3, 0, rowspan=1, columnspan=1, Qt.AlignLeft);
	forecastLayout.addWidget(this.forecTempLbl2, 3, 1, rowspan=1, columnspan=1, Qt.AlignCenter);
	forecastLayout.addWidget(this.forecTempLbl3, 3, 2, rowspan=1, columnspan=1, Qt.AlignRight);

	// A QLabel containing the logo of the BBC is added (weather data feed is obtained from BBC)
	mainLayout.addWidget( this.copyrightLbl, 7, 0, rowspan=1, columnspan=3, Qt.AlignRight );
	
	// Update the weather data 30 minutes
/*
	var timer = new QTimer;
	timer.timeout.connect(this, WeatherWidget.prototype.locChanged);
	timer.start(30000);
*/
  	
    this.setLayout(mainLayout);
    this.autoFillBackground = true;
    this.setWindowTitle(tr("Weather Widget"));
    this.resize(200, 260);
}


// WeatherWidget is of type QWidget
WeatherWidget.prototype = new QWidget();

// A utility function that removes extra whitespaces from the beginning 
// and the end of a string
WeatherWidget.prototype.trimStr = function(str) {
	var trimmed = str.replace(/^\s+|\s+$/g, '');
	return trimmed;
}

// When the city is chosen, the weather conditions for the selected city 
// are fetched.
// http://feeds.bbc.co.uk/weather/feeds/rss/obs/world/CODE.xml
// "San Francisco, California": "USCA0050"  or 6568 
// "Tampere, Finland": "FIXX0031" or 4974
// "London, United Kingdom": "UKXX0318" or 4583
// "Berlin, Germany": 0050
// "Barcelona, Spain": 0042
// "Beijing, China": 0090
// "Kolkata, India": 0145
// "Cape Town, South Africa": 0081
// "Moscow, Russia": 0058
// "Rio De Janeiro, Brazil": 0109
// "Oslo, Norway": 0032
WeatherWidget.prototype.locChanged = function(index) {
    var urlSuffix;
    switch (index) {
        case 0:
            weatherCode = "6568";
            break;
        case 1:
            weatherCode = "4974";
            break;
        case 2:
            weatherCode = "4583";
            break;
        case 3:
            weatherCode = "0050";
            break;
        case 4:
        	weatherCode = "0042";
        	break;
		case 5:
        	weatherCode = "0090";
        	break;
        case 6: 
        	weatherCode = "0145";
        	break;
        case 7:
        	weatherCode = "0081";
        	break;
        case 8:
        	weatherCode = "0058";
        	break;
        case 9: 
        	weatherCode = "0109";
        	break;
        case 10: 
        	weatherCode = "0032";
        	break;
    }
    
	// Get the current weather data
    var url = new QUrl("http://feeds.bbc.co.uk/weather/feeds/rss/obs/world/" + weatherCode +".xml");
	// The reply contains the fetched page that can be further processed.
    this.reqReply = this.downloadFile(url, this.weatherDownloaded);
    
    // Get the weather forecast data
	var forecastUrl = new QUrl("http://feeds.bbc.co.uk/weather/feeds/rss/5day/world/" +weatherCode +".xml");
	// Fetch the forecast 
  	this.forecastReply = this.downloadFile(forecastUrl, this.forecastDownloaded);   
}


// Downloads the file pointed by the url
WeatherWidget.prototype.downloadFile = function(url, callback) {
    // a QNetworkAccessManager handles sending requests and getting replies
    var accessManager = new QNetworkAccessManager();
    // An HTTP request is sent 
    var reqReply = accessManager.get(new QNetworkRequest(url));
    // When the reply is obtained, the execution proceeds to the callback 
    // function.
    reqReply.finished.connect(this, callback);
    // the reply is returned
    return reqReply;
}




// Carries out tasks after the data about current weather conditions has been 
// downloades.
WeatherWidget.prototype.weatherDownloaded = function() {

    // Create a data structure to store the read bytes
    var pageData = new QByteArray();

    // Read the data from the reply to a QByteArray
    pageData = this.reqReply.readAll();

    // New QXmlStreamReader for reading the feed.
    var xml = new QXmlStreamReader(pageData);

    // Read the file, until the end is reached.
    while (!xml.atEnd()) {
        xml.readNext();

        if (xml.isStartElement()) {
            if (xml.name() == "title") {
                this.RSSWeatherDesc = xml.readElementText();
            }
            else if (xml.name() == "description") {
                this.RSSWeatherData = xml.readElementText();
            }
        }
   }

	// Fetch the weather description of the title (sunny, cloydy, rainy etc.)
   	// The description line is of the format: 
   	// Thursday at 0600 GMT: cloudy. -8°C (18°F)
   	var titleData = this.RSSWeatherDesc.split(":");
   	var tmp = titleData[1];
   	var tmp2 = tmp.split(".");
   	var currWeatherDescr = this.trimStr(tmp2[0]);
	// Fetch the name of a correct icon corresponding to the description and 
	// download the file.
	var file = this.getWeather(currWeatherDescr);
   	var reqReply = this.accessManager.get(new QNetworkRequest(new QUrl(file)));
    function loadPic() {
		var data = reqReply.readAll();
		var weatherPic = new QPixmap();
		weatherPic.loadFromData(data, 0, Qt.AutoColor);
		weatherPic = weatherPic.scaled(	128, 128, Qt.KeepAspectRatio, Qt.SmoothTransformation );
		this.descrLbl.setPixmap(weatherPic);
		// Set a tooltip with a weather description
		this.descrLbl.toolTip = tr(currWeatherDescr)
    }
    reqReply.finished.connect(this, loadPic);
	
	// Process the weather data (temperatures).
    var terms = this.RSSWeatherData.split(","); // Split using comma as separator
    var temperatureStr = this.trimStr(terms[0]);
    var words = temperatureStr.split(" ") //split using blank space as delimiter
	// Obtain only the value in centigrades	
	var temp = this.trimStr(words[1]);
	// Use a larger font to show the temperature.
    this.temperature = '<font size=36>' +words[1] +'</font>';
    // Set the temperature to the QLabel and to the tooltip text.
    this.tempLbl.setText(this.temperature);
    this.tempLbl.tooTip = tr(temperatureStr);

	// Wind direction, speed, relative humidity, air pressure, visibility
    this.windDir = this.trimStr(terms[1]);
    this.windSpeed = this.trimStr(terms[2]);
    this.relHumidity = this.trimStr(terms[3]);
    this.pressure = this.trimStr(terms[4]);
    this.pressure +=" " + this.trimStr(terms[5]);
    this.visibility = this.trimStr(terms[6]);
}





// Carries out tasks after the download of the weather forecast.
WeatherWidget.prototype.forecastDownloaded = function() {

    // Create a QByteArray to store the read bytes
    var pageData = new QByteArray();

    // Read the data from the reply to the array
    pageData = this.forecastReply.readAll();

    // New QXmlStreamReader that handles of reading the feed.
    var forecastXML = new QXmlStreamReader(pageData);

	// Forecast data is stored in an array
	this.forecastArray = new Array();

    while (!forecastXML.atEnd()) {
		
	    // Read next item, until the end of the file is reached.
        forecastXML.readNext();

	  	// The weather description line is of the format
		// Thursday: cloudy, Max Temp: -6°C (21°F), Min Temp: -10°C (14°F)
        if (forecastXML.isStartElement()) {
        
            if (forecastXML.name() == "title") {
                this.RSSForecastDesc = forecastXML.readElementText();
                // If the string contains a colon or several ones, 
                // it is of desired format and contains weather forecast data)
                if (this.RSSForecastDesc.indexOf(":") != -1 ) {
		
					var forecItem = new Object();
	                // Use the comma as delimiter
	                var forecastItems = this.RSSForecastDesc.split(",");
					
					var tmp = forecastItems[0];
					var tmp2 = tmp.split(":");
					// Obtain the day and description of the weather
					var day = this.trimStr(tmp2[0]);
					var descr = this.trimStr(tmp2[1]);

					// Fetch the name of a correct icon corresponding to the description
					tmp = forecastItems[1];
					tmp2 = tmp.split(":");
					// Obtain the max temperature
					var temp = this.trimStr(tmp2[1]);
    				var temp2 = temp.split(" "); 
					var maxCelsius = this.trimStr(temp2[0]);

					tmp = forecastItems[2];
					tmp2 = tmp.split(":");
					// Obtain the min temperature
					temp = this.trimStr(tmp2[1]);
					temp2 = temp.split(" ");
					var minCelsius = this.trimStr(temp2[0]);
					
					// Set the attributes (name of the day, description, max 
					// and min temperature in centigrades) for a forecast item
					forecItem.day = day;
					forecItem.descr = descr;
					forecItem.maxC = maxCelsius;
					forecItem.minC = minCelsius;

					// Add the forecast item to the array
					this.forecastArray.push(forecItem);               
                }                
            }
            else if (forecastXML.name() == "description") {
                this.RSSForecastData = forecastXML.readElementText();
            }
           
        }
   	}

	this.forecDayLbl.setText(tr("<font size=2>" +this.forecastArray[0].day +"</font>"));
	var file1 = this.getWeather(this.forecastArray[0].descr);
   	var reqReply = this.accessManager.get(new QNetworkRequest(new QUrl(file1)));
    function loadPic() {
		var data = reqReply.readAll();
		var forecastImg = new QPixmap(file1);
		forecastImg.loadFromData(data, 0, Qt.AutoColor);
		forecastImg = forecastImg.scaled(80, 80, Qt.KeepAspectRatio, Qt.SmoothTransformation);
		this.forecDescrLbl.setPixmap(forecastImg);
		this.forecDescrLbl.toolTip = tr(this.forecastArray[0].descr);
    }
    reqReply.finished.connect(this, loadPic);

	var temperatures = tr("<font size=2>max/min:<br/>" +this.forecastArray[0].maxC +"/"+this.forecastArray[0].minC +"</font>");
	this.forecTempLbl.setText(temperatures);
	this.forecDayLbl2.setText(tr("<font size=2>" +this.forecastArray[1].day +"</font>"));
	var file2 = this.getWeather(this.forecastArray[1].descr);
   	var reqReply2 = this.accessManager.get(new QNetworkRequest(new QUrl(file2)));
    function loadPic2() {
		var data = reqReply2.readAll();
		var forecastImg2 = new QPixmap(file2);
		forecastImg2.loadFromData(data, 0, Qt.AutoColor);
		forecastImg2 = forecastImg2.scaled(	80, 80, Qt.KeepAspectRatio, Qt.SmoothTransformation);
		this.forecDescrLbl2.setPixmap(forecastImg2);
		this.forecDescrLbl2.toolTip = tr(this.forecastArray[1].descr);
    }
    reqReply2.finished.connect(this, loadPic2);

	var temperatures2 = tr("<font size=2>max/min:<br/>" +this.forecastArray[1].maxC +"/"+this.forecastArray[1].minC +"</font>");
	this.forecTempLbl2.setText(temperatures2);

	this.forecDayLbl3.setText(tr("<font size=2>" +this.forecastArray[2].day +"</font>"));
	var file3 = this.getWeather(this.forecastArray[2].descr);
   	var reqReply3 = this.accessManager.get(new QNetworkRequest(new QUrl(file3)));
    function loadPic3() {
		var data = reqReply3.readAll();
		var forecastImg3 = new QPixmap(file3);
		forecastImg3.loadFromData(data, 0, Qt.AutoColor);
		forecastImg3 = forecastImg3.scaled(	80, 80, Qt.KeepAspectRatio, Qt.SmoothTransformation);
		this.forecDescrLbl3.setPixmap(forecastImg3);
		this.forecDescrLbl3.toolTip = tr(this.forecastArray[2].descr);
    }
    reqReply3.finished.connect(this, loadPic3);
	var temperatures3 = tr("<font size=2>max/min:<br/>" +this.forecastArray[2].maxC +"/"+this.forecastArray[2].minC +"</font>");
	this.forecTempLbl3.setText(temperatures3);  
}




// Chooses a suitable icon for a current weather condition from 
// a set of existing ones. The list is by no means complete and due to  
// it "N/A" is shown if a for the program previously unknown weather 
// condition emerges. 
WeatherWidget.prototype.getWeather = function(descr) {

	var baseUrl = "http://lively.cs.tut.fi/images/";
	var fileName = "";
	
	if ( descr == "clear" ) {
		fileName = "Clear.png";
	}
	else if ( descr == "cloudy" ) {
		fileName = "Cloudy.png";
	}
	else if ( descr == "partly cloudy" ) {
		fileName = "Mostly_Cloudy.png";
	}
	else if ( descr == "rainy" ) {
		fileName = "Rain.png";
	}
	else if ( descr == "heavy showers" || descr == "light showers" ) {
		fileName = "Few_Showers.png";
	}
	else if ( descr == "light rain" ) {
		fileName = "Rain.png";
	}
	else if ( descr == "snow" || descr == "heavy snow" ) {
		fileName = "Snow.png";
	}
	else if ( descr == "light snow" || descr == "light snow showers") {
		fileName = "Few_Flurries.png";
	}
	else if ( descr == "sunny" ) {
		fileName = "Sunny.png";
	}
	else if ( descr == "sunny intervals" ) {
		fileName = "Mostly_Sunny.png";
	}
	else if ( descr == "fog" || descr == "misty" ) {
		fileName = "Fog.png";
	}
	// If the weather condition is not available, show a "N/A" image
	else  {
		fileName = "NA.png";
	}
	// Return the base URL and the filename of the icon to be shown.
	return baseUrl + fileName;
}



WeatherWidget.prototype.setPen = function(pen) {
    this.pen = pen;
    this.update();
}




WeatherWidget.prototype.setBrush = function(brush) {
    this.brush = brush;
    this.update();
}




WeatherWidget.prototype.paintEvent = function( /* event */) {
    var pen = new QPen();
    // gradient on the background
    var linearGradient = new QLinearGradient(0, 0, this.width, this.height);
    // ### working around issue with Qt.GlobalColor
    linearGradient.setColorAt(0.0, new QColor(Qt.white));
    linearGradient.setColorAt(0.2, new QColor(83, 130, 161));
    linearGradient.setColorAt(1.0, new QColor(Qt.white));

    var painter = new QPainter();
    painter.begin(this);
    painter.setPen(pen);
    painter.setBrush(new QBrush(linearGradient));
    painter.drawRect(0, 0, this.width, this.height);
    painter.end();
}




/* To use the application in standalone mode, remove the comment around this code
var wdgt = new WeatherWidget();
wdgt.show();

QCoreApplication.exec();
*/

